;;; ACL-COMPAT lite simulates some functionality of Allegro CL package EXCL.
;;;
;;; This is a modified version of Chris Doubles ACL excl wrapper library
;;; As stated in the changelogs of his original this file includes the 
;;; IF* macro placed in the public domain by John Foderaro. 
;;; See: http://www.franz.com/~jkf/ifstar.txt
;;;
;;; This is the header of Chris Doubles original file. (but without Changelog)
;;; ACL excl wrapper library for Corman Lisp - Version 1.1
;;; Copyright (C) 2000 Christopher Double. All Rights Reserved.
;;;
;;; ACL-COMPAT lite Copyright (C) Dmitry Ivanov, 2004-2013.
;;; http://lisp.ystok.ru/
;;; - Clean up, small improvements, and maintenance.

(defpackage :acl-compat.excl
  (:use :common-lisp)
  (:nicknames :excl)
  (:import-from :ystok.library		; should be in the excl package for compatibility
   yl:octets-to-string
   yl:string-to-octets)
  #| +sbcl
  (:import-from :sb-ext
   sb-ext:octets-to-string
   sb-ext:string-to-octets)|#
  (:export
   #:if*				; no longer used by phtml.lisp
   #:*current-case-mode*		; replaced by *xhtml-mode* in phtml.lisp
   #:octets-to-string			; for compatibility
   #:string-to-octets))

(in-package :acl-compat.excl)

(defmacro if* (&rest args)
  (do ((xx (reverse args) (cdr xx))
       (state :init)
       (elseseen nil)
       (totalcol nil)
       (lookat nil nil)
       (col nil))
      ((null xx)
       (cond ((eq state :compl)
              `(cond ,@totalcol))
             (t (error "if*: illegal form ~s" args))))
    (cond ((and (symbolp (car xx))
                (member (symbol-name (car xx)) '("then" "thenret" "else" "elseif")
                        :test #'string-equal))
           (setq lookat (symbol-name (car xx)))))
    
    (cond ((eq state :init)
           (cond (lookat (cond ((string-equal lookat "thenret")
                                (setq col nil
                                      state :then))
                               (t (error "if*: bad keyword ~a" lookat))))
                 (t (setq state :col
                          col nil)
                    (push (car xx) col))))
          ((eq state :col)
           (cond (lookat
                  (cond ((string-equal lookat "else")
                         (cond (elseseen
                                (error "if*: multiples elses")))
                         (setq elseseen t)
                         (setq state :init)
                         (push `(t ,@col) totalcol))
                        ((string-equal lookat "then")
                         (setq state :then))
                        (t (error "if*: bad keyword ~s" lookat))))
                 (t (push (car xx) col))))
          ((eq state :then)
           (cond (lookat
                  (error "if*: keyword ~s at the wrong place " (car xx)))
                 (t (setq state :compl)
                    (push `(,(car xx) ,@col) totalcol))))
          ((eq state :compl)
           (cond ((not (string-equal lookat "elseif"))
                  (error "if*: missing elseif clause ")))
           (setq state :init)))))

(defvar *current-case-mode* :case-insensitive-upper
  "A keyword denoting the current case mode.
 :case-insensitive-upper
   The mode specified in the ANSI specification, the reader converts unescaped
   characters to upper case, e.g. car -> \"CAR\".
 :case-sensitive-lower
   All standard symbols have lower case names. e.g. \"car\". No conversion of
   character case is done on input.")

#+(and unused lispworks)
(defmacro mp::without-scheduling (&body forms)
  `(mp:without-preemption ,@forms))		; Non-SMP LispWorks only

;; The errorset macro is incomplete. It was hacked to get AllegroServe running,
;; but the announce and catch-breaks arguments are ignored.
;; An implementation of the catch-breaks argument will necessarily be
;; implementation-dependent, since ANSI does not allow any
;; program-controlled interception of a break.

#+(and unused tester)
(defmacro errorset (form &optional (announce nil) (catch-breaks nil))
  "The expression form is evaluated and if no errors are signaled,
then the first value returned by the macro will be t 
and the remaining values will be the values returned by form.
If an error is signaled, then the single value nil is returned.

If announce is true, the error message associated with the error
will be printed, otherwise, nothing is printed.

A call to break is not ignored (you do go into a break loop) unless
catch-breaks is true, in which case a warning message is printed and nil is returned"
  (declare (ignore announce catch-breaks))
  `(let* ((ok nil)
          (results (ignore-errors
                     (prog1 (multiple-value-list ,form)
                       (setq ok t)))))
     (if ok
         (apply #'values t results)
         nil)))

(pushnew :acl-compat *features*)
